#ifdef HAVE_CONFIG_H
#include "mysqludf_config.h"
#undef PACKAGE
#undef PACKAGE_NAME
#undef PACKAGE_TARNAME
#undef PACKAGE_STRING
#undef PACKAGE_VERSION
#undef PACKAGE_BUGREPORT
#undef VERSION
#endif
#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif
#ifdef HAVE_STRING_H
#include <string.h>
#endif
#ifdef HAVE_STRINGS_H
#include <strings.h>
#endif

#include <my_global.h>
#include <my_sys.h>
#include <mysql.h>

static inline longlong pad(longlong arg)
{
	if (arg <= 9999LL) {
		arg *= 10000000000LL;
	} else if (arg <= 999999LL) {
		arg *= 100000000LL;
	} else if (arg <= 99999999LL) {
		arg *= 1000000LL;
	} else if (arg <= 9999999999LL) {
		arg *= 10000LL;
	} else if (arg <= 999999999999LL) {
		arg *= 100LL;
	}
	return arg;
}

static inline longlong now(void)
{
	time_t now = time(NULL);
	struct tm *act_time;
#if defined(HAVE_LOCALTIME_R) && defined(_REENTRANT)
	struct tm tmp_time;
	
	localtime_r(&now, &tmp_time);
	act_time = &tmp_time;
#else
	act_time = localtime(&now);
#endif
	return 
			((longlong) act_time->tm_year + 1900LL) * 10000000000LL +
			((longlong) act_time->tm_mon  +    1LL) * 100000000LL +
			((longlong) act_time->tm_mday)          * 1000000LL +
			((longlong) act_time->tm_hour)          * 10000LL +
			((longlong) act_time->tm_min)           * 100LL +
			((longlong) act_time->tm_sec);
}

my_bool humanage_init(UDF_INIT *init, UDF_ARGS *args, char *message)
{
	switch (args->arg_count) {
		case 2:
			args->arg_type[1] = INT_RESULT;
		case 1:
			args->arg_type[0] = INT_RESULT;
			break;
		default:
			strncpy(message, "HUMANAGE() accepts 1 or 2 parameters", MYSQL_ERRMSG_SIZE);
			return 1;
	}
	
	init->maybe_null = 1;
	
	return 0;
}

void humanage_deinit(UDF_INIT *init)
{
}

longlong humanage(UDF_INIT *init, UDF_ARGS *args, char *is_null, char *error)
{
	longlong age, arg1, arg2;
	
	switch (args->arg_count) {
		case 2:
			if (args->args[1]) {
				break;
			}
		case 1:
			if (args->args[0]) {
				break;
			}
			*is_null = (char) 1;
			return 0;
	}
	
	if (args->args[0]) {
		arg1 = pad(*((longlong *) args->args[0]));
		if (args->arg_count == 2 && args->args[1]) {
			arg2 = pad(*((longlong *) args->args[1]));
		} else {
			arg2 = now();
		}
	} else {
		arg1 = now();
		arg2 = pad(*((longlong *) args->args[1]));
	}
	
	if (arg1 > arg2 && arg2 > 0LL) {
		age = arg1 - arg2;
	} else {
		age = arg2 - arg1;
	}
	
	return age / 10000000000LL;
}
